@@ -27,6 +27,7 @@
 #include <locale.h>
 #include <gdm/gdm-client.h>
 
+#include "cc-common-language.h"
 #include "gis-assistant.h"
 
 #define GIS_TYPE_DRIVER_MODE (gis_driver_mode_get_type ())
@@ -83,6 +84,8 @@ struct _GisDriverPrivate {
   GisDriverMode mode;
   UmAccountMode account_mode;
   gboolean small_screen;
+
+  locale_t locale;
 };
 typedef struct _GisDriverPrivate GisDriverPrivate;
 
@@ -113,6 +116,12 @@ gis_driver_finalize (GObject *object)
 
   g_clear_object (&priv->user_account);
 
+  if (priv->locale != (locale_t) 0)
+    {
+      uselocale (LC_GLOBAL_LOCALE);
+      freelocale (priv->locale);
+    }
+
   G_OBJECT_CLASS (gis_driver_parent_class)->finalize (object);
 }
 
@@ -158,12 +167,46 @@ gis_driver_get_assistant (GisDriver *driver)
   return priv->assistant;
 }
 
-void
-gis_driver_set_user_language (GisDriver *driver, const gchar *lang_id)
+static void
+gis_driver_real_locale_changed (GisDriver *driver)
 {
   GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
+  GtkTextDirection direction;
+
+  direction = gtk_get_locale_direction ();
+  gtk_widget_set_default_direction (direction);
+
+  rebuild_pages (driver);
+  gis_assistant_locale_changed (priv->assistant);
+}
+
+static void
+gis_driver_locale_changed (GisDriver *driver)
+{
+  g_signal_emit (G_OBJECT (driver), signals[LOCALE_CHANGED], 0);
+}
+
+void
+gis_driver_set_user_language (GisDriver *driver, const gchar *lang_id, gboolean update_locale)
+{
+  GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
+
   g_free (priv->lang_id);
   priv->lang_id = g_strdup (lang_id);
+
+  cc_common_language_set_current_language (lang_id);
+
+  if (update_locale)
+    {
+      locale_t locale = newlocale (LC_MESSAGES_MASK, lang_id, (locale_t) 0);
+      uselocale (locale);
+
+      if (priv->locale != (locale_t) 0 && priv->locale != LC_GLOBAL_LOCALE)
+        freelocale (priv->locale);
+      priv->locale = locale;
+
+      gis_driver_locale_changed (driver);
+    }
 }
 
 const gchar *
@@ -255,25 +298,6 @@ gis_driver_hide_window (GisDriver *driver)
   gtk_widget_hide (GTK_WIDGET (priv->main_window));
 }
 
-static void
-gis_driver_real_locale_changed (GisDriver *driver)
-{
-  GisDriverPrivate *priv = gis_driver_get_instance_private (driver);
-  GtkTextDirection direction;
-
-  direction = gtk_get_locale_direction ();
-  gtk_widget_set_default_direction (direction);
-
-  rebuild_pages (driver);
-  gis_assistant_locale_changed (priv->assistant);
-}
-
-void
-gis_driver_locale_changed (GisDriver *driver)
-{
-  g_signal_emit (G_OBJECT (driver), signals[LOCALE_CHANGED], 0);
-}
-
 GisDriverMode
 gis_driver_get_mode (GisDriver *driver)
 {
@@ -504,7 +528,7 @@ gis_driver_startup (GApplication *app)
 
   gtk_widget_show (GTK_WIDGET (priv->assistant));
 
-  gis_driver_set_user_language (driver, setlocale (LC_MESSAGES, NULL));
+  gis_driver_set_user_language (driver, setlocale (LC_MESSAGES, NULL), FALSE);
 
   prepare_main_window (driver);
   rebuild_pages (driver);
